/*
 * Screen Saver - Lockup Desktop replacement for OS/2 and eComStation systems
 * Copyright (C) 2004-2005 Doodle
 *
 * Contact: doodle@dont.spam.me.scenergy.dfmk.hu
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <process.h>

#define INCL_DOS
#define INCL_WIN
#define INCL_ERRORS
#include <os2.h>

#include "SSDPMS.h"
#include "snap/graphics.h"

int        bInitialized;

int        iCurrentDPMSState = SSDPMS_STATE_ON;
GA_devCtx *SNAPDeviceContext = NULL;

//#define DEBUG_LOGGING
#ifdef DEBUG_LOGGING
void AddLog(char *pchMsg)
{
  FILE *hFile;

  hFile = fopen("e:\\ssdpms.log", "at+");
  if (hFile)
  {
    fprintf(hFile, "%s", pchMsg);
    fclose(hFile);
  }
}
#endif


SSDPMSDECLSPEC int SSDPMSCALL SSDPMS_GetCapabilities(int *piCapabilities)
{
  GA_DPMSFuncs DPMSFuncs;
  N_int16      supportedStates;

  // Check parameters!
  if (!piCapabilities)
    return SSDPMS_ERROR_INVALIDPARAMETER;

  if (!bInitialized)
    return SSDPMS_ERROR_SERVICEUNAVAILABLE;

#ifdef DEBUG_LOGGING
  AddLog("[SSDPMS_GetCapabilities] : Calling GA_queryFunctions and DPMSdetect\n");
#endif

  // Detect if the DPMS interface is active
  DPMSFuncs.dwSize = sizeof(DPMSFuncs);
  if ((!(GA_queryFunctions(SNAPDeviceContext,
                         GA_GET_DPMSFUNCS,
                         &DPMSFuncs))) ||
      (!(DPMSFuncs.DPMSdetect(&supportedStates))) ||
      (supportedStates == 0))
  {
    // DPMS services are not available!
#ifdef DEBUG_LOGGING
    AddLog("[SSDPMS_GetCapabilities] : DPMS services are not available (SNAP)\n");
#endif

    *piCapabilities = SSDPMS_STATE_ON;
    return SSDPMS_NOERROR;
  }

  // Return supported DPMS modes

#ifdef DEBUG_LOGGING
  AddLog("[SSDPMS_GetCapabilities] : Returning supported modes\n");
#endif

  *piCapabilities = SSDPMS_STATE_ON;

  if (supportedStates & DPMS_standby)
    *piCapabilities |= SSDPMS_STATE_STANDBY;

  if (supportedStates & DPMS_suspend)
    *piCapabilities |= SSDPMS_STATE_SUSPEND;

  if (supportedStates & DPMS_off)
    *piCapabilities |= SSDPMS_STATE_OFF;

  return SSDPMS_NOERROR;
}

SSDPMSDECLSPEC int SSDPMSCALL SSDPMS_SetState(int iDPMSState)
{
  int iSupportedStates;
  GA_DPMSFuncs DPMSFuncs;
  N_int32      stateToSet;

  if (!bInitialized)
    return SSDPMS_ERROR_SERVICEUNAVAILABLE;

  if (SSDPMS_GetCapabilities(&iSupportedStates)!=SSDPMS_NOERROR)
  {
    // Eeeek, error querying capabilities!
    // State setting failed!
#ifdef DEBUG_LOGGING
    AddLog("[SSDPMS_SetState] : Could not get capabilities!\n");
#endif
    return SSDPMS_ERROR_INTERNALERROR;
  }

  if ((iSupportedStates & iDPMSState)==0)
  {
#ifdef DEBUG_LOGGING
    AddLog("[SSDPMS_SetState] : Requested state not supported!\n");
#endif
    return SSDPMS_ERROR_STATENOTSUPPORTED;
  }

  // Okay, it seems that the requested DPMS state is supported.
  // Let's try to set it, then!

  if (iCurrentDPMSState == iDPMSState)
  {
#ifdef DEBUG_LOGGING
    AddLog("[SSDPMS_SetState] : Shortcut exit.\n");
#endif

    // Shortcut, we're already in the requested state!
    return SSDPMS_NOERROR;
  }

#ifdef DEBUG_LOGGING
  AddLog("[SSDPMS_SetState] : Query DPMS functions from SNAP\n");
#endif

  // Query DPMS functions from SNAP
  DPMSFuncs.dwSize = sizeof(DPMSFuncs);
  if (!GA_queryFunctions(SNAPDeviceContext,
                         GA_GET_DPMSFUNCS,
                         &DPMSFuncs))
  {
#ifdef DEBUG_LOGGING
    AddLog("[SSDPMS_SetState] : Error querying DPMS functions from SNAP!\n");
#endif
    return SSDPMS_ERROR_INTERNALERROR;
  }

  switch (iDPMSState)
  {
    case SSDPMS_STATE_OFF:
      stateToSet = DPMS_off;
      break;
    case SSDPMS_STATE_STANDBY:
      stateToSet = DPMS_standby;
      break;
    case SSDPMS_STATE_SUSPEND:
      stateToSet = DPMS_suspend;
      break;
    case SSDPMS_STATE_ON:
    default:
      stateToSet = DPMS_on;
      break;
  }

#ifdef DEBUG_LOGGING
  AddLog("[SSDPMS_SetState] : Calling DPMSsetState()\n");
#endif

  // If all goes well, this should set dpms state on all heads of the
  // primary video device.
  DPMSFuncs.DPMSsetState(stateToSet);

#ifdef DEBUG_LOGGING
  AddLog("[SSDPMS_SetState] : Saving current state, and returning\n");
#endif

  // Save the current state.
  iCurrentDPMSState = iDPMSState;

  return SSDPMS_NOERROR;
}

SSDPMSDECLSPEC int SSDPMSCALL SSDPMS_GetState(int *piDPMSState)
{
  // Check parameters!
  if (!piDPMSState)
    return SSDPMS_ERROR_INVALIDPARAMETER;

  if (!bInitialized)
    return SSDPMS_ERROR_SERVICEUNAVAILABLE;

  // Return current DPMS state
  *piDPMSState = iCurrentDPMSState;

  return SSDPMS_NOERROR;
}


static int SNAP_Initialize()
{
  // Load SNAP driver for primary display
  // Note that we do not handle the case, when there are
  // multiple video cards in the system.
  // If that should also be handled, we should load a driver for
  // all the cards, and call DPMSsetState() for all the drivers
  // in SSDPMS_SetState().

  if ((SNAPDeviceContext = GA_loadDriver(0, false))==NULL)
  {
    // Ooops, could not load SNAP driver for primary display.
#ifdef DEBUG_LOGGING
    AddLog("[SNAP_Initialize] : Error in GA_loadDriver(0, false)!\n");
#endif
    // Something went wrong. Return error!
    return 0;
  }

  // Okay, SNAP is with us! :)
  return 1;
}

static void SNAP_Uninitialize()
{
  if (SNAPDeviceContext)
  {
    // Uninitialize SNAP driver
    GA_unloadDriver(SNAPDeviceContext);
    SNAPDeviceContext = NULL;
  }
}

SSDPMSDECLSPEC int SSDPMSCALL SSDPMS_Initialize(void)
{
  HMODULE     hModSDDGRADD;

  if (bInitialized) return SSDPMS_NOERROR;

  // First of all, check if SNAP is present!
  if (DosQueryModuleHandle("SDDGRADD.DLL", &hModSDDGRADD) != NO_ERROR)
  {
    // SNAP not installed
#ifdef DEBUG_LOGGING
    AddLog("[SSDPMS_Initialize] : SNAP is not installed.\n");
#endif
    return SSDPMS_ERROR_SERVICEUNAVAILABLE;
  }

  // Ok, SNAP seems to be installed, let's see if we can initialize it!
  if (!SNAP_Initialize())
  {
    // Could not initialize SNAP!
    // It might be that the user doesn't have SNAP drivers, or
    // something else happened...
#ifdef DEBUG_LOGGING
    AddLog("[SSDPMS_Initialize] : Could not initialize SNAP!\n");
#endif
    return SSDPMS_ERROR_SERVICEUNAVAILABLE;
  }

  // Okay, SNAP is here.
  // Now initialize other local variables
  iCurrentDPMSState = SSDPMS_STATE_ON;
  bInitialized = TRUE;

#ifdef DEBUG_LOGGING
  AddLog("[SSDPMS_Initialize] : Service initialized.\n");
#endif

  return SSDPMS_NOERROR;
}

SSDPMSDECLSPEC int SSDPMSCALL SSDPMS_Uninitialize(void)
{
  if (bInitialized)
  {
    SNAP_Uninitialize();
    bInitialized = FALSE;

#ifdef DEBUG_LOGGING
    AddLog("[SSDPMS_Uninitialize] : Service uninitialized.\n");
#endif
    return SSDPMS_NOERROR;
  } else
    return SSDPMS_ERROR_SERVICEUNAVAILABLE;
}


//---------------------------------------------------------------------
// LibMain
//
// This gets called at DLL initialization and termination.
//---------------------------------------------------------------------
unsigned _System LibMain(unsigned hmod, unsigned termination)
{
  if (termination)
  {
    // Cleanup!
#ifdef DEBUG_LOGGING
    AddLog("[LibMain] : TERMINATE DLL (start)\n");
#endif


    // If the client disconnects, and leaves the monitor in saving mode,
    // we should set it back to ON state!
    if (iCurrentDPMSState!=SSDPMS_STATE_ON)
      SSDPMS_SetState(SSDPMS_STATE_ON);

    // Okay, now drop SNAP functions, in case the client forgot it!
    SSDPMS_Uninitialize();

#ifdef DEBUG_LOGGING
    AddLog("[LibMain] : TERMINATE DLL (done)\n");
#endif

    return 1; // Return success!
  } else
  {
    // Startup!

#ifdef DEBUG_LOGGING
    AddLog("[LibMain] : INITIALIZE DLL (start)\n");
#endif

    // Now initialize local variables
    iCurrentDPMSState = SSDPMS_STATE_ON;
    SNAPDeviceContext = NULL;
    bInitialized = FALSE;

#ifdef DEBUG_LOGGING
    AddLog("[LibMain] : INITIALIZE DLL (done)\n");
#endif

    // All done, we're completely initialized!
    return 1;
  }
}

